#!/usr/local/bin/ruby

#
# xr2pop, Copyright (C) 2001 Andreas Bolka <andreas.bolka@gmx.net>
#
# provides a simple XML-RPC to POP3 gateway, closely
# mapping POP3 actions to XML-RPC callable methods
#
# -----
#
# COPYRIGHT AND PERMISSION NOTICE
#
# Copyright (c) 2001 Andreas Bolka
#
# All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, and/or sell copies of the Software, and to permit persons
# to whom the Software is furnished to do so, provided that the above
# copyright notice(s) and this permission notice appear in all copies of
# the Software and that both the above copyright notice(s) and this
# permission notice appear in supporting documentation.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
# OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
# HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
# INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
# FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
# WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
# Except as contained in this notice, the name of a copyright holder
# shall not be used in advertising or otherwise to promote the sale, use
# or other dealings in this Software without prior written authorization
# of the copyright holder.
#
# -----
#

require 'xmlrpc/server'
require 'net/pop' 

# --- config section

listen_ip = "0.0.0.0"
listen_port = 3030

# --- handler definition

class SessionHandler
	def initialize( s )
		@sessions = Hash[]
		@server_instance = s
	end

	def remove( obj )
		@sessions.delete( @sessions.index( obj ) )
	end

	def login( user, pass, server, port )
		pop = Net::POP3.new( server, port )
		do_login( pop, user, pass )
	end

	def login_apop( user, pass, server, port )
		pop = Net::APOP.new( server, port )
		do_login( pop, user, pass )
	end

	private

	def do_login( pop, user, pass )
		begin

			pop.start( user, pass )

			sid = pop.id.to_s
			h = POPHandler.new( self, pop )

			@sessions[ sid ] = h
			@server_instance.add_handler( "gateways.pop." + sid, h )

			return sid

		rescue Net::ProtoAuthError => e
			raise XMLRPC::FaultException.new( 101, "Failed to POP authenticate!" )
		end
	end
end

class POPHandler

	private

		def message( msgnum )
			if @pop.mails[msgnum-1] == nil
				raise XMLRPC::FaultException.new( 201, "Message #{msgnum} does not exist." )
			end

			@pop.mails[msgnum-1]
		end

	public

		def initialize( mgr, pop_session )
			@session_manager = mgr
			@pop = pop_session
		end

		def logout
			@pop.finish
			@session_manager.remove( self )

			true
		end

		def stat
			sum = 0
			@pop.mails.each do |msg| sum += msg.size end

			{ "count" => @pop.mails.length, "size" => sum }
		end

		def list
			list = Hash[]
			@pop.mails.each_index do |i|
				list[ i+1 ] = @pop.mails[i].size
			end

			list
		end

		def dele( msgnum )
			message( msgnum ).delete

			true
		end

		def retr( msgnum )
			# warning: messages will be only encoded using LF instead of CRLF
			message( msgnum ).all
		end

		def retr64( msgnum )
			# here the messages should properly decode in CRLF seperated 
			# streams as required per RFC 822
			XMLRPC::Base64.encode( retr( msgnum ).gsub( /\n/, "\r\n" ) )
		end

		def uidl( msgnum )
			message( msgnum ).uidl
		end

		def top( msgnum, lines )
			message( msgnum ).top( lines )
		end
end

# ---- startup

server = XMLRPC::Server.new( listen_port, listen_ip )
server.add_handler( "gateways.pop.session", SessionHandler.new( server ) )
server.serve

# $Id: xr2pop.rb,v 1.1 2001-08-23 22:55:47+02 earl Exp $
